/*
	* Copyright (c) 2021-11 by ELECTRONIC ASSEMBLY <technik@lcd-module.de>
	*
	* Bluetooth implementation for demoproject with EA uniTFTs
	*
	* This file is free software; you can redistribute it and/or modify
	* it under the terms of either the GNU General Public License version 2
	* or the GNU Lesser General Public License version 2.1, both as
	* published by the Free Software Foundation.
*/

#include "debug.h"
#include <Arduino.h>
#include "bt_keyboard.h"
#include <sdkconfig.h>
#include "uniTFTDesigner_BT.h"
#include "src/BT/bluetooth.h"
#include "src/SmallProtocol/uniTFT.h"
#include "src/WiFi/wifi_ea.h"

#define MAX_BT_NAMELEN 20
static RTC_NOINIT_ATTR  byte after_Restart_BT_Mode;
static RTC_NOINIT_ATTR  char after_Restart_BT_SerialName[MAX_BT_NAMELEN];
static RTC_NOINIT_ATTR  char after_Restart_BT_ManufacturerName[MAX_BT_NAMELEN];

/*----------------------------
	Func: setup
	Desc: Decide ESP32 bluetooth mode after restart
	Vars: reset reason
	Ret : ---
------------------------------*/
void BT_Keyboard::setup(esp_reset_reason_t reason) 
{
	if ((reason != ESP_RST_DEEPSLEEP) && (reason != ESP_RST_SW)) //Reset reason: power on or hardware reset
	{
		after_Restart_BT_Mode = MODE_OFF; //Bluetooth off
		after_Restart_BT_SerialName[0]=0;
		after_Restart_BT_ManufacturerName[0] = 0;
		BT_serial.setup(NULL); //only initialize reset port for bluetooth
	}
	else
	{
		connection_mode = after_Restart_BT_Mode; 
		if(connection_mode == MODE_BT_SERIAL) //Reset reason: sw-reset to switch to serial mode
		{
			if(strlen(after_Restart_BT_SerialName))
			{
				BT_serial.setup(after_Restart_BT_SerialName); //Bluetooth device name
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.printf("Restart with array name: %s\n", after_Restart_BT_SerialName);
				#endif
			}
			else
			{
				sprintf(strbuf, "%s", "EA uniTFTs Direct"); 
				BT_serial.setup(strbuf); //Bluetooth device name
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.println("Restart mit hardname");
				#endif
			}
		}
		else if(connection_mode == MODE_BT_KEYBOARD) //Reset reason: sw-reset to switch to keyboard mode
		{
			WiFi_EA.disconnect(); //WiFi off, bluetooh and wifi does not work together.
			connection_mode = MODE_BT_KEYBOARD;
			kb.setName(after_Restart_BT_SerialName);
			kb.setManufacturerName(after_Restart_BT_ManufacturerName);
			kb.begin();
			was_BT_Keyboard = true;
		}
	}
	
  	
	
}

/*----------------------------
	Func: BT_off
	Desc: Bluetooth off
	Vars: ---
	Ret : ---
------------------------------*/
void BT_Keyboard::BT_off()
{
	connection_mode = MODE_OFF;
	BT_serial.end();
	kb.end();
}

/*----------------------------
	Func: isConnectedAsSerial
	Desc: checks if bluetooth is connected as serial port
	Vars: ---
	Ret : bool: true on connected
------------------------------*/
boolean BT_Keyboard::isConnectedAsSerial()
{
	return connection_mode == MODE_BT_SERIAL;
}

/*----------------------------
	Func: checkTask
	Desc: checks if there is something to do for bluetooth
	Vars: ---
	Ret : ---
------------------------------*/
void BT_Keyboard::checkTask() 
{
	if(connection_mode == MODE_BT_SERIAL && digitalRead(BT_RESETPIN)==1) //uniTFT wants to quit direct bluetooth connection
		BT_off();
	if(connection_mode == MODE_OFF) //Bluetooth off?
		return; //nothing to do
	else if(connection_mode == MODE_BT_KEYBOARD) //connected as keyboard?
	{   
		if(kb_callFirstScreen == true && kb.isConnected()) //first time bluetooth is connected call screen in uniTFT
		{
			kb_callFirstScreen = false;
			sprintf(strbuf, "#MRT %d, 2\n",  sc_BT_Welcome_bt_BT_main); //call screen
			uniTFT.sendDataPacket(strbuf, strlen(strbuf));
		}
		if(kb.isConnected() && kb_connectIcon == 0) //set connection icon to the correct status
		{
			kb_connectIcon = 1;
			sprintf(strbuf, "#TCS 2, %d\n",  sc_BT_Background_ic_connection); //is connected
			uniTFT.sendDataPacket(strbuf, strlen(strbuf));
		}
		else  if(!kb.isConnected() && kb_connectIcon == 1) //set connection icon to the correct status
		{
		kb_connectIcon = 0;
		sprintf(strbuf, "#TCS 1, %d\n",  sc_BT_Background_ic_connection); //not connected
		uniTFT.sendDataPacket(strbuf, strlen(strbuf));
		}
	}
	else if(connection_mode == MODE_BT_SERIAL) //connected as serial port?
		BT_serial.serial(); //check if data is to send/receive
}

/*----------------------------
	Func: decodeCommands
	Desc: Decode commands from uniTFTs concerning bluetooth keyboard commands
	Vars: command
	Ret : ---
------------------------------*/
void BT_Keyboard::decodeCommands(byte command)
{
	byte lb, hb, mk;
	switch(command)
	{
		case 'P': //Press Key
		case 'R': //Release Key
		case 'W': //Write Key
		{
			lb = uniTFT.getByte(); hb = uniTFT.getByte(); mk = uniTFT.getByte(); //get lowByte, highbyte and media key
			uniTFT.getByte(); //highest byte unused
			#if (DBG_ON && DBG_BLUETOOTH)			
				Serial.print("Key MK: "); Serial.print(mk);  Serial.printf(" hb: 0x%02X lb; 0x%02X\r\n", hb, lb);
			#endif
			if(mk == 0x01 && kb.isConnected()) //Media key
			{
				MediaKeyReport rep = {hb,lb};
				if(command == 'W')
					kb.write(rep);
				else if(command == 'P')
					kb.press(rep);
				else if(command == 'R')
					kb.release(rep);					
			}
			else if(kb.isConnected()) //Normal key
			{
				if(command == 'W')
					kb.write(lb);
				else if(command == 'P')
					kb.press(lb);
				else if(command == 'R')
					kb.release(lb);
			}
		}break;
		case 'C': //Bluetooth connect
		{
			lb = uniTFT.getByte(); //bluetooth mode: 0=disconnect 1=Keyboard 2=serial port (direct)
			if(lb == 0 || (kb.isConnected() && lb >= 1)) //disconnect or connect and already connected
			{
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.println("Stop BT!");
				#endif
				BT_off();
				if(lb)
					delay(2000);
				WiFi_EA.reconnect();
			}
			if(lb == 1)
			{
				WiFi_EA.disconnect(); //WiFi off, not together with bluetooth
				if(was_BT_Keyboard) //keyboard was on, but new connection is needed, so restart
				{
					after_Restart_BT_Mode = MODE_BT_KEYBOARD;
					uniTFT.getString(after_Restart_BT_SerialName,MAX_BT_NAMELEN);
					after_Restart_BT_SerialName[MAX_BT_NAMELEN-1] = 0;
					uniTFT.getString(after_Restart_BT_ManufacturerName,MAX_BT_NAMELEN);
					after_Restart_BT_ManufacturerName[MAX_BT_NAMELEN-1] = 0;
					ESP.restart();
				}
				connection_mode = MODE_BT_KEYBOARD;
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.println("Starting BT!");
				#endif
				uniTFT.getString(strbuf);
				kb.setName(strbuf); //Bluetooth device name
				#if(DBG_ON && DBG_BLUETOOTH)
					Serial.println(strbuf);
				#endif
				uniTFT.getString(strbuf);
				kb.setManufacturerName(strbuf); //Bluetooth manufacturer name
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.println(strbuf);
				#endif
				kb_callFirstScreen = true; //on connection established call first screen
				was_BT_Keyboard = true;
				kb.begin(); //start keyboard
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.printf("kb.begin end: call first screen %d on connect\n", kb_callFirstScreen);
				#endif
			}
			if(lb == 2)
			{
				if(was_BT_Keyboard) //keyboard was on, but new connection is needed, so restart in serial mode
				{
					after_Restart_BT_Mode = MODE_BT_SERIAL;
					uniTFT.getString(after_Restart_BT_SerialName,MAX_BT_NAMELEN);
					after_Restart_BT_SerialName[MAX_BT_NAMELEN-1] = 0;
					ESP.restart();
				}
				connection_mode = MODE_BT_SERIAL;
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.println("Starting BT serial work!");
				#endif
				uniTFT.getString(strbuf);
				BT_serial.setup(strbuf); //Bluetooth device name
				#if (DBG_ON && DBG_BLUETOOTH)
					Serial.printf("Direct BT ready - Virtual COM %s\r\n", strbuf);
				#endif				
			}
		}break;
		default:
		break;
	}
}
